#include "ControlledMotor.h"

ControlledMotor::ControlledMotor() { this->type = Thing::ControlledMotorType; }

ControlledMotor::ControlledMotor(Motor *motor, Encoder *encoder)
    : ControlledMotor() {
  this->motor = motor;
  this->encoder = encoder;
  // this->rotationDirection = Direction::Forward;
}

#include <Arduino.h>

void ControlledMotor::SetTargetSpeed(float speed) {
  this->targetSpeed = speed;
  // this->rotationDirection =
  //     (targetSpeed < 0) ? Direction::Reverse : Direction::Forward;
  // this->direction = (targetSpeed < 0) ? Motor::Direction::CounterClockwise
  //                                     : Motor::Direction::Clockwise;
}

void ControlledMotor::Update(float currentTimeMs) {
  actualSpeed = encoder->GetRevolutionsPerSecond(currentTimeMs);
  if (this->targetSpeed < 0)
    actualSpeed = -actualSpeed;

  // Serial.print("direction ");
  // Serial.print(this->direction);
  // Serial.print("  actualSpeed ");
  // Serial.println(actualSpeed);
  // Serial.print("  target ");
  // Serial.println(this->targetSpeed);

  float error = targetSpeed - velocity;

  float timeStep = currentTimeMs - lastUpdateTime;
  float acceleration = error * timeStep * pidP; // Just P is used at this moment
  motor->SetTargetSpeed(
      this->targetSpeed); // + acceleration); // or something like that
  this->lastUpdateTime = currentTimeMs;
}

float ControlledMotor::GetActualSpeed() { return actualSpeed; }

bool ControlledMotor::Drive(float distance) {
  if (!driving) {
    targetDistance = distance;
    startDistance = encoder->GetDistance();
    driving = true;
  }
  float totalDistance = encoder->GetDistance() - startDistance;
  bool completed = totalDistance > targetDistance;
  return completed;
}